

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include <math.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <linux/i2c-dev.h>
#include "bme280.h"


/* Delay function. Uses select() with no file-descriptors */
static void delay(int milliseconds)
{
	fd_set emp;
	struct timeval tx;

	memset(&tx, 0, sizeof(struct timeval));

	tx.tv_sec = milliseconds / 1000;
	tx.tv_usec = (milliseconds * 1000) % 1000000;
	
	FD_ZERO(&emp);
	select(0, &emp, &emp, &emp, &tx);
}


/* Read and pigeonhole calibration values: 
	0x88-0xA1 into dig_T and digP
	0xe1-0xF0 into dig_H */

int BME280_readcal(int fd, struct BME280_Cal *Cx)
{
	int nr; 
	uint8_t arr[44];

	memset(arr, 0, 44); 

	i2c_smbus_write_byte(fd, 0x88);
	nr = read(fd, arr, 26);

	Cx->dig_T1 = (arr[ 1] << 8) | arr[ 0]; 
	Cx->dig_T2 = (arr[ 3] << 8) | arr[ 2]; 
	Cx->dig_T3 = (arr[ 5] << 8) | arr[ 4]; 

	Cx->dig_P1 = (arr[ 7] << 8) | arr[ 6]; 
	Cx->dig_P2 = (arr[ 9] << 8) | arr[ 8]; 
	Cx->dig_P3 = (arr[11] << 8) | arr[10]; 
	Cx->dig_P4 = (arr[13] << 8) | arr[12]; 
	Cx->dig_P5 = (arr[15] << 8) | arr[14]; 
	Cx->dig_P6 = (arr[17] << 8) | arr[16]; 
	Cx->dig_P7 = (arr[19] << 8) | arr[18]; 
	Cx->dig_P8 = (arr[21] << 8) | arr[20]; 
	Cx->dig_P9 = (arr[23] << 8) | arr[22]; 

	Cx->dig_H1 = arr[0xA1-0x88]; 

	memset(arr, 0, 44); 

	i2c_smbus_write_byte(fd, 0xe1);
	nr = read(fd, arr, 10);

	Cx->dig_H2 = (arr[0xe2-0xe1] << 8) | arr[0xe1-0xe1]; 
	Cx->dig_H3 = arr[0xe3-0xe1]; 
	Cx->dig_H6 = arr[0xe7-0xe1]; 

	// printf("%s(%d): 0x%x 0x%x 0x%x\n", __FILE__, __LINE__, arr[0xe4-0xe1], arr[0xe5-0xe1], arr[0xe6-0xe1]);  

	Cx->dig_H4 = (arr[0xe4-0xe1] << 4) | (arr[0xe5-0xe1] & 0x0f);

	// printf("%s(%d): dig_H4 = 0x%x\n", __FILE__, __LINE__, Cx->dig_H4);

	Cx->dig_H5 = (arr[0xe6-0xe1] << 4) | ( (arr[0xe5-0xe1] >> 4) & 0x0f);

	// printf("%s(%d): dig_H5 = 0x%x\n", __FILE__, __LINE__, Cx->dig_H5);

	return(nr);
}


/* Read raw adc values from registers f7-fe into the three
	values for Temperature, Pressure and Humidity. 

		0xf7 - press_msb - P[19:12]
		0xf8 - press_lsb - P[11:4]
		0xf9 - press_xlsb- P[3:0] 0 0 0 0
		0xfa - temp_msb  - T[19:12]
		0xfb - temp_lsb  - T[11:4]
		0xfc - temp_xlsb - T[3:0] 0 0 0 0
		0xfd - hum_msb   - H[15:8]
		0xfe - hum_lsb   - H[7:0]
*/
int BME280_readvalues(int fd, int32_t *Traw, int32_t *Praw, int32_t *Hraw) 
{
	int nr; 
	uint8_t arr[10];
	uint32_t vx;

	memset(arr, 0, 10);

	/* We will want to block read from 0xf7-0xfe */ 
	i2c_smbus_write_byte(fd, 0xf7);
	nr = read(fd, arr, 10);

	vx = 0;
	vx |= arr[0];
	vx <<= 8;
	vx |= arr[1];
	vx <<= 8;
	vx |= arr[2];
	vx >>= 4;
	*Praw = vx;

	vx = 0; 
	vx |= arr[3];
	vx <<= 8;
	vx |= arr[4];
	vx <<= 8;
	vx |= arr[5];
	vx >>= 4;
	*Traw = vx;

	vx = 0; 
	vx |= arr[6];
	vx <<= 8;
	vx |= arr[7];
	*Hraw = vx;

	return(nr);
}


/* Returns temperature in DegC, double precision. Output value of "51.23" equals 51.23 DegC. 
	Also establises the value of t_fine for P and H calculations.

	adc_T is the 20 bit value read from registers 0xfa,0xfb,0xfc

		0xf7 - press_msb - P[19:12]
		0xf8 - press_lsb - P[11:4]
		0xf9 - press_xlsb- P[3:0] 0 0 0 0
		0xfa - temp_msb  - T[19:12]
		0xfb - temp_lsb  - T[11:4]
		0xfc - temp_xlsb - T[3:0] 0 0 0 0
		0xfd - hum_msb   - H[15:8]
		0xfe - hum_lsb   - H[7:0]
*/

double BME280_compensate_T_degC( int32_t adc_T, int32_t *pt_fine, struct BME280_Cal *Cx)
{
	double var1, var2, T;

	var1  = ((( double) adc_T)/16384.0 - (( double) Cx->dig_T1)/1024.0) * (( double)Cx->dig_T2); 
	var2  = (((( double)adc_T)/131072.0 - (( double)Cx->dig_T1)/8192.0) * 
		((( double)adc_T)/131072.0 - (( double) Cx->dig_T1)/8192.0)) * (( double)Cx->dig_T3);
	*pt_fine = ( int32_t)(var1 + var2); 
	T  = (var1 + var2) / 5120.0;
	return T;
}


/* Returns pressure in Pa as double. Output value of  96386.2  equals 96386.2 Pa = 963.862 hPa
	adc_P is the 20 bit value read from registers 0xf7,0xf8,0xf9
*/

double BME280_compensate_P_Pascal( int32_t adc_P, int32_t t_fine, struct BME280_Cal *Cx)
{
	double var1, var2, p;

	var1 = (( double)t_fine/2.0) -  64000.0;
	var2 = var1 * var1 * (( double)Cx->dig_P6) / 32768.0;
	var2 = var2 + var1 * (( double)Cx->dig_P5) * 2.0;
	var2 = (var2/4.0)+((( double)Cx->dig_P4) * 65536.0);
	var1 = ((( double)Cx->dig_P3) * var1 * var1 / 524288.0 + (( double)Cx->dig_P2) * var1) / 524288.0;
	var1 = (1.0 + var1 / 32768.0)*(( double)Cx->dig_P1);

	if (var1 == 0.0)
	{
		return 0; // avoid exception caused by division by zero
	}

	p = 1048576.0 - ( double)adc_P;
	p = (p - (var2 / 4096.0)) * 6250.0 / var1;
	var1 = (( double)Cx->dig_P9) * p * p / 2147483648.0;
	var2 = p * (( double)Cx->dig_P8) / 32768.0;
	p = p + (var1 + var2 + (( double)Cx->dig_P7)) / 16.0;
	return p;
}


/* Returns humidity in % rH as as double .  Output value of "46.332" represents 46.332 % rH
	adc_H is the 16 bit value read from registers 0xfd,0xfe
*/ 

double BME280_compensate_H_percent(int32_t adc_H, int32_t t_fine, struct BME280_Cal *Cx)
{
	double var_H;

	var_H = ((( double)t_fine) - 76800.0);

	var_H = (adc_H - ((( double)Cx->dig_H4) * 64.0 + (( double)Cx->dig_H5) / 16384.0 * var_H)) *
	((( double)Cx->dig_H2) / 65536.0 * (1.0 + (( double)Cx->dig_H6) / 67108864.0 * var_H *
	(1.0 + (( double)Cx->dig_H3) / 67108864.0 * var_H)));

	var_H = var_H * (1.0 - (( double)Cx->dig_H1) * var_H / 524288.0);

	if (var_H > 100.0)
		var_H = 100.0;
	else if (var_H < 0.0)
		var_H = 0.0;

	return var_H;
}

/* Make the full reading. */ 

int BME280_Reading(char *i2cdev, int ba, 
		double *Ta, 
		double *Pa, 
		double *Ha, 
		char *info, size_t infomax, int verbose)
{
	int s; 
	int fd;
	struct BME280_Cal CA;
	int32_t tfine;
	int32_t Traw, Praw, Hraw;
	char errinfo[100];

	ba &= 0x1;
	ba |= 0x76; /* Can only be 0x76 or 0x77 */ 

	fd = open(i2cdev, O_RDWR);
	if(fd == -1) 
	{ 
		strerror_r(errno, errinfo, 100); 
		snprintf(info, infomax, "Open %s: %s", i2cdev, errinfo); 
		return(-1); 
	}

	s = ioctl(fd, I2C_SLAVE, ba); 
	if(s == -1) 
	{ 
		snprintf(info, infomax, "Set slave address 0x%x: %s", ba, errinfo); 
		strerror_r(errno, errinfo, 100); 
		return(-1); 
	}

	/* Reset */ 
	s = i2c_smbus_write_byte_data(fd, 0xe0, 0xb6);

	/* open and prep the device. */ 
	
	s = i2c_smbus_read_byte_data(fd, 0xd0);
	if(verbose) 
		printf("%s(%d): Ident = 0x%x\n", __FILE__, __LINE__, s); 

	s = i2c_smbus_read_byte_data(fd, 0xf5);
	if(verbose) 
		printf("%s(%d): config = 0x%x\n", __FILE__, __LINE__, s); 

	s = i2c_smbus_write_byte_data(fd, 0xf2, 0x01); // 0xf2, 0x01 // 00000001

	s = i2c_smbus_write_byte_data(fd, 0xf4, 0x25); // 0xf4, 0x25 // 00100101

	s = i2c_smbus_read_byte_data(fd, 0xf2);
	if(verbose) 
		printf("%s(%d): Ctrl_hum = 0x%x\n", __FILE__, __LINE__, s); 

	s = i2c_smbus_read_byte_data(fd, 0xf4);
	if(verbose) 
		printf("%s(%d): Ctrl_meas = 0x%x\n", __FILE__, __LINE__, s); 

	s = i2c_smbus_read_byte_data(fd, 0xf3);
	if(verbose) 
		printf("%s(%d): status = 0x%x\n", __FILE__, __LINE__, s); 

	delay(100);

	s = i2c_smbus_read_byte_data(fd, 0xf3);
	if(verbose) 
		printf("%s(%d): status = 0x%x\n", __FILE__, __LINE__, s); 

	s = i2c_smbus_read_byte_data(fd, 0xf3);
	if(verbose) 
		printf("%s(%d): status = 0x%x\n", __FILE__, __LINE__, s); 

	s = BME280_readcal(fd, &CA);

	if(verbose) 
		printf("%s(%d): T: %u %d %d\n", __FILE__, __LINE__, CA.dig_T1, CA.dig_T2, CA.dig_T3); 

	if(verbose) 
		printf("%s(%d): P: %u %d %d %d %d %d %d %d %d\n", __FILE__, __LINE__, CA.dig_P1, CA.dig_P2, CA.dig_P3, 
			CA.dig_P4, CA.dig_P5, CA.dig_P6, CA.dig_P7, CA.dig_P8, CA.dig_P9); 

	if(verbose) 
		printf("%s(%d): H: %u %d %u %d %d %u\n", __FILE__, __LINE__, CA.dig_H1, CA.dig_H2, CA.dig_H3, 
			CA.dig_H4, CA.dig_H5, CA.dig_H6); 

	s = BME280_readvalues(fd, &Traw, &Praw, &Hraw); 

	*Ta = BME280_compensate_T_degC(Traw, &tfine, &CA); 
	*Pa = BME280_compensate_P_Pascal(Praw, tfine, &CA); 
	*Ha = BME280_compensate_H_percent(Hraw, tfine, &CA); 

	snprintf(info, infomax, "Ta = %G C\nPa = %G mbar\nHa = %G %%\n", *Ta, *Pa/100.0, *Ha); 
	close(fd);

	return(s);
}


/* Ambient Partial Pressure */
double calc_PPTamb(double Tamb)
{
	double A = 8.1332;
	double B = 1762.39;
	double C = 235.66;

	double E = B / (Tamb + C);
	double pptamb = pow(10.0, A-E); 

	return( pptamb );
}

/* Dew point Temperature from Partial Pressure and Rel Humidity. */
double calc_Td(double RHamb, double PPTamb)
{
	double A = 8.1332;
	double B = 1762.39;
	double C = 235.66;

	double N = log10(RHamb * PPTamb / 100.0) - A; 

	double Td = - ( (B / N) + C);

	return(Td);
}









