
/* Bme280 -- Bosch Temperature, Humidity, and Pressure combined sensor */

#ifndef BME280HINCLUDED
#define BME280HINCLUDED

/* I2c address = 0x76 or 0x77
	Default is 0x77 (SDO to H) 
	6disc uses 0x77	
	7disc uses 0x76 as there is another device at 0x77
*/ 


/* 
	Registers:
		0x88...0xa1 - calibration values 0...25
		0xe1...0xf0 - calibration values 26...41
		0xd0 - id - ident, read 0x60
		0xe0 - reset - Reset-command, write 0xB6 to reset the device

		0xf2 - ctrl_hum  - 0 0 0 0 0 OSRS_H[2:0]
		0xf3 - status    - 0 0 0 0 Measuring 0 0 IM_Update
		0xf4 - ctrl_meas - OSRS_T[2:0] OSRS_P[2:0] Mode[1:0]
		0xf5 - config    - T_AB[2:0] Filter[2:0] 0 SPI3W_EN

		0xf7 - press_msb - P[19:12]
		0xf8 - press_lsb - P[11:4]
		0xf9 - press_xlsb- P[3:0] 0 0 0 0
		0xfa - temp_msb  - T[19:12]
		0xfb - temp_lsb  - T[11:4]
		0xfc - temp_xlsb - T[3:0] 0 0 0 0
		0xfd - hum_msb   - H[15:8]
		0xfe - hum_lsb   - H[7:0]
*/ 

#define IDENT 0xd0

#define RESET_REG 0xe0
#define RESET_CMD 0xB6

#define CTRL_HUM 0xf2
		
/* 
	0xf2 ctrl_hum: Humidity oversampling setting:  (write this before writing ctrl_meas)
	OSRS_H[2:0]

	Oversampling values, for OSRS_H, OSRS_P, and OSRS_T:
	000 - Not done, skipped, output is 0x8000 for H, 0x80000 for P and T
	001 - oversampling x1 
	010 - oversampling x2 
	011 - oversampling x4 
	100 - oversampling x8 
	101 - oversampling x16 
	110 - oversampling x16
	111 - oversampling x16
*/

#define STATUS_REG 0xf3

/* 
	0xf3 Status register:
	Measuring: 1 when conversion is in progress, 0 when done
	Update: 1 when NVM data is being copied, 0 when not doing that
*/

#define CTRL_MEAS 0xf4

/*
	0xf4 ctrl_meas:  (write this after writing ctrl_hum) 
	OSRS_T[2:0] - temperature oversampling (values like the ones for humidity)
	OSRS_P[2:0] - pressure oversampling (values like the ones for humidity)
	Mode:
		00 - Sleep
		01 - Forced mode
		10 - Forced mode
		11 - Normal mode
*/

#define CONFIG_REG 0xf5

/*
	0xf5 config:
	T_SB[2:0] Standby time
		000 - 0.5 ms
		001 - 62.5 ms
		010 - 125 ms
		011 - 250 ms
		100 - 500 ms
		101 - 1000 ms
		110 - 10 ms
		111 - 20 ms

	Filter[2:0] IIR filter parameter
		000 - filter is off
		001 - 2
		010 - 4
		011 - 8
		1xx - 16

	SPI3W_EN - set to 1 to enable SPI, 0 to enable i2c
*/ 


/* To make the measurements, first run the Temperature conversion function, as this
	generates a t_fine temperature value used for the other two measurements */ 

/* Calibration values: */ 
struct BME280_Cal
{
	uint16_t dig_T1;  // = from 0x88, 0x89
	int16_t  dig_T2;  // = from 0x8a, 0x8b
	int16_t  dig_T3;  // = from 0x8c, 0x8d

	uint16_t dig_P1;  // = from 0x8e, 0x8f
	int16_t  dig_P2;  // = from 0x90, 0x91
	int16_t  dig_P3;  // = from 0x92, 0x93
	int16_t  dig_P4;  // = from 0x94, 0x95
	int16_t  dig_P5;  // = from 0x96, 0x97
	int16_t  dig_P6;  // = from 0x98, 0x99
	int16_t  dig_P7;  // = from 0x9a, 0x9b
	int16_t  dig_P8;  // = from 0x9c, 0x9d
	int16_t  dig_P9;  // = from 0x9e, 0x9f

	uint8_t	dig_H1;  // = from 0xA1
	int16_t dig_H2;  // = from 0xe1, 0xe2
	uint8_t	dig_H3;  // = from 0xe3
	uint8_t	dig_H6;  // = from 0xe7

	/* 12-bit quantities:  */ 

	int16_t dig_H4; //  = from 0xe4 [11:4], 0xe5[3:0]
	
		// 0xe4: 11 10 9 8 7 6 5 4 
		// 0xe5:  x  x x x 3 2 1 0

	int16_t dig_H5; //  = from 0xe5[7:4]=[3:0],  0xe6[11:4]

		// 0xe5:  3  2 1 0 x x x x
		// 0xe6: 11 10 9 8 7 6 5 4 
};

/* Read and pigeonhole calibration values: 
	0x88-0xA1 into dig_T and digP
	0xe1-0xF0 into dig_H */

int BME280_readcal(int fd, struct BME280_Cal *Cx);


/* Read raw adc values from registers f7-fe into the three
	values for Temperature, Pressure and Humidity. 

		0xf7 - press_msb - P[19:12]
		0xf8 - press_lsb - P[11:4]
		0xf9 - press_xlsb- P[3:0] 0 0 0 0
		0xfa - temp_msb  - T[19:12]
		0xfb - temp_lsb  - T[11:4]
		0xfc - temp_xlsb - T[3:0] 0 0 0 0
		0xfd - hum_msb   - H[15:8]
		0xfe - hum_lsb   - H[7:0]
*/
int BME280_readvalues(int fd, int32_t *Traw, int32_t *Praw, int32_t *Hraw); 


/* Returns temperature in DegC, double precision. Output value of "51.23" equals 51.23 DegC. 
	Also establises the value of t_fine for P and H calculations.  
	adc_T is the 20 bit value read from registers 0xfa,0xfb,0xfc
*/ 

double BME280_compensate_T_degC( int32_t adc_T, int32_t *pt_fine, struct BME280_Cal *Cx);


/* Returns pressure in Pa as double. Output value of  96386.2  equals 96386.2 Pa = 963.862 hPa
	adc_P is the 20 bit value read from registers 0xf7,0xf8,0xf9
*/

double BME280_compensate_P_Pascal( int32_t adc_P, int32_t t_fine, struct BME280_Cal *Cx);


/* Returns humidity in % rH as as double .  Output value of "46.332" represents 46.332 % rH
	adc_H is the 16 bit value read from registers 0xfd,0xfe
*/ 

double BME280_compensate_H_percent(int32_t adc_H, int32_t t_fine, struct BME280_Cal *Cx);

/* Make the full reading. */ 

int BME280_Reading(char *i2cdev, int ba, 
		double *Ta, 
		double *Pa, 
		double *Ha, 
		char *info, size_t infomax, int verbose);

/* Ambient Partial Pressure */
double calc_PPTamb(double Tamb);

/* Dew point Temperature from Partial Pressure and Rel Humidity. */
double calc_Td(double RHamb, double PPTamb);


#endif




